using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using System.Threading;
using ADSCF;

namespace WatchdogTest
{
    public delegate void ChangeTxtFieldDelegate(TextBox tb, string st);     // declare type

    public partial class WatchdogMain : Form
    {

        public ChangeTxtFieldDelegate UpdateTextFieldFunction;      // declare function ptr of that type

        private bool running;
        private Thread RemainingTimeThread;
        private Thread ServiceThread;

        AutoResetEvent autoStopEvent;

        ulong TimeoutValue = 0;
        ulong TimeoutPortion = 0;


        public WatchdogMain()
        {
            InitializeComponent();

            UpdateTextFieldFunction = new ChangeTxtFieldDelegate(UpdateTextField);  // point function to local code

            autoStopEvent = new AutoResetEvent(false);
        }

        private void btnRunTest_Click(object sender, EventArgs e)
        {
            ulong Time = 0;
            bool Enabled = false;

            if (rdWatchdogGetStatus.Checked)
            {
                if (ADSAPI.ADSIsEnabledWatchDog(ref Enabled))
                {

                    if (Enabled)
                    {
                        UpdateTextField(txtTestStatus, "Watchdog is enabled.");
                    }
                    else
                    {
                        UpdateTextField(txtTestStatus, "Watchdog is not enabled.");
                    }
                }
                else
                {
                    UpdateTextField(txtTestStatus, "Could not get watchdog status.");
                }

            }
            else if (rdWatchdogStart.Checked)
            {
                if (running)
                {
                    UpdateTextField(txtTestStatus, "Watchdog already running.");
                }
                else if (ADSAPI.ADSStartWatchDog())
                {
                    UpdateTextField(txtTestStatus, "Watchdog started.");

                    running = true;

                    ServiceThread = new Thread(new ThreadStart(ServiceWatchdogThread));
                    ServiceThread.Start();

                    RemainingTimeThread = new Thread(new ThreadStart(UpdateRemainingTimeThread));
                    RemainingTimeThread.Start();

                }
                else
                {
                    UpdateTextField(txtTestStatus, "Could not start watchdog.");
                }
            }
            else if (rdWatchdogGetTime.Checked)
            {
                if (ADSAPI.ADSGetWatchDogTime(ref Time))
                {
                    UpdateTextField(txtTestStatus, "Watchdog time is " + Time + ".");
                }
                else
                {
                    UpdateTextField(txtTestStatus, "Could not get watchdog time.");
                }

            }
            else if (rdWatchdogSetTime1.Checked)
            {
                if (running)
                {
                    if (!ADSAPI.ADSServiceWatchdog())
                    {
                        UpdateTextField(txtTestStatus, "Could not service watchdog before setting new time.");
                    }
                }

                Time = 15000;
                if (ADSAPI.ADSSetWatchDogTime(ref Time))
                {
                    UpdateTextField(txtTestStatus, "Watchdog time set to " + Time + ".");

                    if (running)
                    {
                        if (!ADSAPI.ADSServiceWatchdog())
                        {
                            UpdateTextField(txtTestStatus, "Could not service watchdog after setting new time.");
                        }

                        TimeoutPortion = Time / 4;
                    }
                }
                else
                {
                    UpdateTextField(txtTestStatus, "Could not set watchdog time.");
                }
            }
            else if (rdWatchdogSetTime2.Checked)
            {
                if (running)
                {
                    if (!ADSAPI.ADSServiceWatchdog())
                    {
                        UpdateTextField(txtTestStatus, "Could not service watchdog before setting new time.");
                    }
                }

                Time = 25000;
                if (ADSAPI.ADSSetWatchDogTime(ref Time))
                {
                    UpdateTextField(txtTestStatus, "Watchdog time set to " + Time + ".");

                    if (running)
                    {
                        if (!ADSAPI.ADSServiceWatchdog())
                        {
                            UpdateTextField(txtTestStatus, "Could not service watchdog after setting new time.");
                        }

                        TimeoutPortion = Time / 4;
                    }

                }
                else
                {
                    UpdateTextField(txtTestStatus, "Could not set watchdog time.");
                }
            }
            else if (rdWatchdogService.Checked)
            {
                if (ADSAPI.ADSServiceWatchdog()) 
                {
                    UpdateTextField(txtTestStatus, "Watchdog serviced.");
                }
                else
                {
                    UpdateTextField(txtTestStatus, "Could not service watchdog.");
                }

            }
            else if (rdWatchdogStop.Checked)
            {
                if (running)
                {
                    running = false;
                    
                    RemainingTimeThread.Abort();
                    UpdateTextField(txtTimeRemaining, "0");

                    autoStopEvent.Set();
                    
                    ServiceThread.Abort();

                }

                if (!ADSAPI.ADSStopWatchDogNoReset())
                {
                    UpdateTextField(txtTestStatus, "Could not stop watchdog.");
                }
                else
                {
                    UpdateTextField(txtTestStatus, "Watchdog stopped but requires a reset to take effect.");
                }
            }

            else if (rdWatchdogStopReset.Checked)
            {
                if (running)
                {
                    running = false;

                    RemainingTimeThread.Abort();
                    UpdateTextField(txtTimeRemaining, "0");

                    autoStopEvent.Set();

                    ServiceThread.Abort();

                }

                if (!ADSAPI.ADSStopWatchDogWithReset())
                {
                    UpdateTextField(txtTestStatus, "Could not stop watchdog.");
                }
                else
                {
                    UpdateTextField(txtTestStatus, "Watchdog stopped.");
                }
            }

        }

        /// <summary>
        /// Delegate to Update Textbox
        /// </summary>
        /// <param name="lb">txtField to Update</param>
        /// <param name="text">Text to set</param>
        private void UpdateTextField(TextBox txtField, string text)
        {
            txtField.Text = text;
            txtField.Update();
        }

        protected override void OnClosing(CancelEventArgs e)
        {
            base.OnClosing(e);
            if (running)
            {
                running = false;

                ServiceThread.Abort();

                RemainingTimeThread.Abort();
            }

        }

        private void btnExit_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void ServiceWatchdogThread() 
        {

            if (ADSAPI.ADSGetWatchDogTime(ref TimeoutValue))
            {
                TimeoutPortion = (TimeoutValue / 4);
            }
            else
                this.Invoke(UpdateTextFieldFunction, txtTestStatus, "Could not get watchdog time.");

            while (running)
            {
                if (autoStopEvent.WaitOne((int)TimeoutPortion, false) == false) // timeout, portion small enough to fit in int
                {
                    if (!ADSAPI.ADSServiceWatchdog())
                    {
                        this.Invoke(UpdateTextFieldFunction, txtTestStatus, "Could not service watchdog.");
                    }
                }
                else // stop event signaled
                {
                    Thread.Sleep(1000);
                    
                    break;                 
                }
            }
        }

        private void UpdateRemainingTimeThread()
        {
            ulong RemainingTime = 0;

            while (running)
            {
                Thread.Sleep(2000);

                if (ADSAPI.ADSGetWatchDogTimeRemaining(ref RemainingTime))
                {
                    this.Invoke(UpdateTextFieldFunction, txtTimeRemaining, RemainingTime.ToString());
                }
                else
                {
                    UpdateTextField(txtTimeRemaining, "0");
                }
            }
        }
    }
}